% =========================================================================
% This program is used to test implications of alternative
% theories across steady states. The main difference with stationary_main.m 
% is that this one takes entry costs as given and solves for prices.
%
% The only experiment not considered here is the increase in returns to scale. 
% That one can be found in the 'returns_to_scale_counterfactual' folder.
% =========================================================================
%% -------------------------------------------------------------------------
% Initialize parameters and other model primitives
% -------------------------------------------------------------------------
clc
clear all
close all

mstar = 100 ;

% Fixed Parameters 
beta      = 1/1.04; % Discount rate
alpha     = 0.64 ; % Labor share
cf_l      = 2.298950284374937;    % Fixed cost - intercept% 
sigma0    = 1.331137767741511;    % Variance of startup distribution (\sigma_G^2)
mu_l      = -2.431373086987380;   % Long-run AR1 mean of high type  
omega_h   = 0.359430723369395;    % Share of high type firms
s0        = -4.344376541584754;  % Mean of startup distribution (\mu_g)
sigma_eps = 0.245520815536363;    % Variance of AR1 innovations
mu_h      = -1.436111629482697 ;   % Long-run AR1 mean of high type
cf_h      = 24.308026243791222;    % Fixed cost - intercept

ns = 100 ; % number of points in grid
tol = 1e-6 ; % Specify tolerance

% Save as a structure
params.alpha = alpha ;
params.beta = beta ;
params.mstar = mstar ;
params.sigma0 = sigma0 ;
params.ns = ns ;
params.tol = tol ;
params.omega_h = omega_h;
params.cf_h = cf_h;
params.cf_l = cf_l;

% Experiment Parameters: 1978 Benchmark
g_1978 =  0.026513188 ;            % 1978 labor force growth
ce_1978 = 0.012815402736800;       % 1978 entry cost
rho_1978   = 0.984150757243253;    % 1978 Persistence  of AR1

% Experiment Parameter: Alternative values
g_2014   = 0.007747463 ; % 2014 labor force growth
ce_exp1  = 0.2;          % Higher entry cost
rho_exp2 = 0.9905;       % Lower diffusion

%%
%------------------------------------------
% Experiment 1: Recover 1978 Bench
%------------------------------------------

g = g_1978;
ce = ce_1978;
rho = rho_1978;

% Productivity grid using Tauchen method
[svec, F_l, F_h] = tauchen2(mu_l, mu_h, rho, sigma_eps, ns) ; 

F_l = transpose(F_l) ; % Transpose so that column sum to one
F_h = transpose(F_h) ; % Transpose so that column sum to one

% Startup productivity
step = svec(2) - svec(1) ;
G = normcdf(svec + step/2, s0, sigma0) ;
G(2:end-1) = G(2:end-1) - G(1:end-2) ;
G(end) = 1 - sum(G(1:end-1)) ;

% Create a structure that we want to transfer to functions
params.ce = ce;
params.g = g ;
params.F_h = F_h;
params.F_l = F_l;
params.G = G ;
params.svec = svec ;

% Solve for alternative experiment, store results in alteqbm
alteqbm_bench = stationary_alt(params) ; 

%%
%------------------------------------------
% Experiment 2: Recover 2014 Pop Growth
%------------------------------------------

g = g_2014;
ce = ce_1978;
rho = rho_1978;

% Productivity grid using Tauchen method
[svec, F_l, F_h] = tauchen2(mu_l, mu_h, rho, sigma_eps, ns) ; 

F_l = transpose(F_l) ; % Transpose so that column sum to one
F_h = transpose(F_h) ; % Transpose so that column sum to one

% Startup productivity
step = svec(2) - svec(1) ;
G = normcdf(svec + step/2, s0, sigma0) ;
G(2:end-1) = G(2:end-1) - G(1:end-2) ;
G(end) = 1 - sum(G(1:end-1)) ;

% Create a structure that we want to transfer to functions
params.ce = ce;
params.g = g ;
params.F_h = F_h;
params.F_l = F_l;
params.G = G ;
params.svec = svec ;

% Solve for alternative experiment, store results in alteqbm
alteqbm_lpop = stationary_alt(params) ; 

%%
%------------------------------------------
% Experiment 3: Higher Entry Cost
%------------------------------------------

g = g_1978;
ce = ce_exp1;
rho = rho_1978;

% Productivity grid using Tauchen method
[svec, F_l, F_h] = tauchen2(mu_l, mu_h, rho, sigma_eps, ns) ; 

F_l = transpose(F_l) ; % Transpose so that column sum to one
F_h = transpose(F_h) ; % Transpose so that column sum to one

% Startup productivity
step = svec(2) - svec(1) ;
G = normcdf(svec + step/2, s0, sigma0) ;
G(2:end-1) = G(2:end-1) - G(1:end-2) ;
G(end) = 1 - sum(G(1:end-1)) ;

% Create a structure that we want to transfer to functions
params.ce = ce;
params.g = g ;
params.F_h = F_h;
params.F_l = F_l;
params.G = G ;
params.svec = svec ;

% Solve for alternative experiment, store results in alteqbm
alteqbm_hentry = stationary_alt(params) ; 

%%
%------------------------------------------
% Experiment 4: Decrease in Mean Reversion
%------------------------------------------

g = g_1978;
ce = ce_1978;
rho = rho_exp2;

% Productivity grid using Tauchen method
[svec, F_l, F_h] = tauchen2(mu_l, mu_h, rho, sigma_eps, ns) ; 

F_l = transpose(F_l) ; % Transpose so that column sum to one
F_h = transpose(F_h) ; % Transpose so that column sum to one

% Startup productivity
step = svec(2) - svec(1) ;
G = normcdf(svec + step/2, s0, sigma0) ;
G(2:end-1) = G(2:end-1) - G(1:end-2) ;
G(end) = 1 - sum(G(1:end-1)) ;

% Create a structure that we want to transfer to functions
params.ce = ce;
params.g = g ;
params.F_h = F_h;
params.F_l = F_l;
params.G = G ;
params.svec = svec ;

% Solve for alternative experiment, store results in alteqbm
alteqbm_lrow = stationary_alt(params) ; 

%%
save alteqbm_bench alteqbm_bench
save alteqbm_lpop alteqbm_lpop
save alteqbm_lrow alteqbm_lrow
save alteqbm_hentry alteqbm_hentry