# This file cleans the data from BDS and saves the resulting data in csv files, so they can be used by other programs
# We will clean 
# (1) aggregate firm data (bds_f_all_release) 
# (2) data by firm ages 0 to 25+ (bds_f_age0to25plus_release)
# (3) data by firm sector (bds_f_sic_release)
# (4) data by firm age x sector (bds_f_age0to25plus_sic_release)

# Data cleaning includes defining variables that we need, imputing concentration and combining data for firm-age groups


rm(list = ls())
library(data.table)
library(stringr)


# Read datasets 
# Aggregate data
dt.all <- fread("./raw files/bds_f_all_release.csv")
names(dt.all) <- tolower(names(dt.all))

# Data by firm size
dt.sz <- fread("./raw files/bds_f_sz_release.csv") # read emp by size categories
names(dt.sz) <- tolower(names(dt.sz))
szcat <- unique(dt.sz$fsize)

# Data by firm age
dt.age <- fread("./raw files/bds_f_age_release.csv")
names(dt.age) <- tolower(names(dt.age))
agecat <- sort(unique(dt.age$fage4))

# Data by firm age x size
dt.agesz <- fread("./raw files/bds_f_agesz_release.csv")
names(dt.agesz) <- tolower(names(dt.agesz))


# ------------------------------------------------------------------------------
# Aggregate calculations
# ------------------------------------------------------------------------------
# Calculate total number of employees/total number of firms for aggregate
dt.all[, avgfsize := emp/firms]
dt.all[, exitrate:= firmdeath_firms/shift(firms)*100]

# ------------------------------------------------------------------------------
# Calculate concentration
# ------------------------------------------------------------------------------
dt.emp250plus <- dt.sz[fsize %in% szcat[7:12], .(emp250plus=sum(emp)), by=year2] # sum emp in 250+
dt.all <- merge(dt.all, dt.emp250plus) # merge with original dataset
dt.all[, empshare250plus := emp250plus/emp*100] # calculate fraction


# ------------------------------------------------------------------------------
# Entry rate
# ------------------------------------------------------------------------------
dt.entrants <- dt.age[fage4 %in% agecat[1], .(entrants = firms, emp0yrolds = emp), by=year2]
dt.all <- merge(dt.all, dt.entrants)
dt.all[, entry_rate := entrants/shift(firms)*100]
dt.all[, empshare0yrolds := emp0yrolds/emp*100]

# ------------------------------------------------------------------------------
# Calculate share of 11+ firms
# Saves it as share11plus.csv
# ------------------------------------------------------------------------------
dt.11plus <- dt.age[fage4 %in% agecat[8:12], .(firms11plus = sum(firms), emp11plus=sum(emp)), by=year2]
dt.all <- merge(dt.all, dt.11plus)
dt.all[, firmshare11plus := firms11plus/firms*100]
dt.all[, empshare11plus:= emp11plus/emp*100]


fwrite(dt.all, "./cleaned files/bds_f_all_cleaned.csv")


# ------------------------------------------------------------------------------
# Variables by firm age
# ------------------------------------------------------------------------------
# Calculate total number of employees/total number of firms for each size category
dt.age[, avgfsize := emp/firms]
dt.age[, exitrate:= firmdeath_firms/firms*100]

# Calculate Concentration by Age
# Break firms into age-size categories
temp <- dt.agesz[, .(year2, fage4, fsize, firms, emp)]
# calculate number of firms size 250- by age
temp.250minus <- temp[(fsize%in% szcat[1:6]), .(emp.250minus=sum(emp)), by=.(year2, fage4)]
# calculate number of firms size 250+ by age
temp.250plus <- temp[!(fsize%in% szcat[1:6]), .(emp.250plus=sum(emp)), by=.(year2, fage4)]
# combine with data on total firms
dt.age <- merge(dt.age, temp.250minus, by=c("year2", "fage4"), all=TRUE)
dt.age <- merge(dt.age, temp.250plus, by=c("year2", "fage4"), all=TRUE)
# calculate share
dt.age[, empshare250minus := emp.250minus/emp*100]
dt.age[, empshare250plus := emp.250plus/emp*100]
dt.age[, empshare250plus.imputed := 100 - empshare250minus]

# Create new data set with ages 0 to 25 only
dt.age0to25 <- dt.age
# Drop years for which the age bin is not completely populated
dt.age0to25 <- dt.age0to25[!(fage4 == "g) 6 to 10" & year2 < 1987)]
dt.age0to25 <- dt.age0to25[!(fage4 == "h) 11 to 15" & year2 < 1992)]
dt.age0to25 <- dt.age0to25[!(fage4 == "i) 16 to 20" & year2 < 1997)]
dt.age0to25 <- dt.age0to25[!(fage4 == "j) 21 to 25" & year2 < 2002)]
dt.age0to25 <- dt.age0to25[!(fage4 == "k) 26+")]
dt.age0to25 <- dt.age0to25[!(fage4 == "l) Left Censored")]

# Choose firms in 26+ and Left Censored categories
# Drop values before 2002, no 26+ firms before that
temp <- dt.age[fage4 %in% c("k) 26+", "l) Left Censored")][year2 >= 2002, .(year2, fage4, firms, estabs, emp, avgfsize, 
                                                                            exitrate,empshare250plus.imputed)]
                                                                            
temp[, firmwts := firms/sum(firms), by=year2]
temp[, empwts := emp/sum(emp), by=year2]
temp2 <- temp[, .(firms=sum(firms), emp=sum(emp), 
                  avgfsize = weighted.mean(avgfsize, w=firmwts),
                  exitrate = weighted.mean(exitrate, w=firmwts),
                  empshare250plus.imputed = weighted.mean(empshare250plus.imputed, w=empwts)),
              by=year2]
temp2[, fage4 := "k) Above 25"]

# Merge, with all column names as in smaller data.table
temp3 <- merge(temp2, dt.age0to25, all=TRUE, by=names(temp2))
dt.age0to25plus <- temp3[, .(firms, estabs, emp, avgfsize, exitrate, 
                             empshare250plus, empshare250plus.imputed), by=.(year2, fage4)] 
setorder(dt.age0to25plus, year2, fage4)


# Write all the age files
fwrite(dt.age0to25plus, "./cleaned files/bds_f_age0to25plus_cleaned.csv")



#------------------------------------------------------------------------------------------------------
# Clean data on sectors, which will be used for regressions
#------------------------------------------------------------------------------------------------------
# Start with sector level data, then age-sector, then age-size-sector
dt.sic <- fread("./raw files/bds_f_sic_release.csv")
names(dt.sic) <- tolower(names(dt.sic))
sic.codes <- dt.sic[, unique(sic1)]
sector.names <- c("Agriculture", "Mining", "Construction", "Manufacturing", "Transportation", "Wholesale Trade", "Retail Trade", "Finance, Insurance and Real Estate", "Services")
sector.names.clean <- str_replace_all(sector.names, ",", "")
sector.names.clean <- str_replace_all(sector.names.clean, " ", "_")
# Create data.table with SIC codes and corresponding sector names
temp.sic <- data.table(sic1=sic.codes, sector_name=sector.names)
# Merge dataset with bds.data.sic
dt.sic <- merge(dt.sic, temp.sic, by="sic1", all=TRUE)

# Calculate total number of employees/total number of firms for each age category
dt.sic[, avgfsize := emp/firms]
dt.sic[, exitrate:= firmdeath_firms/firms*100]

dt.sz_sic <- fread("./raw files/bds_f_szsic_release.csv")
names(dt.sz_sic) <- tolower(names(dt.sz_sic))
dt.sz_sic <- merge(dt.sz_sic, temp.sic, by="sic1", all=TRUE)
dt.emp250minus_sic <- dt.sz_sic[fsize %in% szcat[1:6], .(emp250minus=sum(emp)), by=.(year2, sector_name)] # sum emp in 250-
dt.emp250plus_sic <- dt.sz_sic[fsize %in% szcat[7:12], .(emp250plus=sum(emp)), by=.(year2, sector_name)] # sum emp in 250+
temp <- merge(dt.emp250minus_sic, dt.emp250plus_sic)

dt.sic <- merge(dt.sic, temp, by=c("year2", "sector_name")) # merge with original dataset
dt.sic[, empshare250plus.direct := emp250plus/emp*100, by=sector_name] # calculate fraction
dt.sic[, empshare250plus.imputed := 100*(1 - emp250minus/emp), by=sector_name] # calculate fraction

dt.sic[, empshare250plus:= empshare250plus.direct]
dt.sic[is.na(empshare250plus)==TRUE, empshare250plus:= empshare250plus.imputed]

# Load age files
dt.age_sic <- fread("./raw files/bds_f_agesic_release.csv")
names(dt.age_sic) <- tolower(names(dt.age_sic))
# Merge dataset with bds.data.sic
dt.age_sic <- merge(dt.age_sic, temp.sic, by="sic1", all=TRUE)

# Load size files so we can calculate concentration
dt.age_sz_sic <- fread("./raw files/bds_f_agesz_sic_release.csv")
names(dt.age_sz_sic) <- tolower(names(dt.age_sz_sic))
# Merge dataset with bds.data.sic to match sector names with sic codes
dt.age_sz_sic <- merge(dt.age_sz_sic, temp.sic, by="sic1", all=TRUE)


dt.0yr <- dt.age_sic[fage4 %in% agecat[1], .(entrants = firms), by=.(year2, sic1)]
dt.sic <- merge(dt.sic, dt.0yr, by=c("year2","sic1"))
dt.sic[, entry_rate := entrants/shift(firms)*100, by=sic1]

dt.11plus <- dt.age_sic[fage4 %in% agecat[8:12], .(firms11plus = sum(firms), emp11plus=sum(emp)), by=.(year2,sic1)]
dt.sic <- merge(dt.sic, dt.11plus, by=c("year2","sic1"))
dt.sic[, ':='(firmshare11plus=firms11plus/firms*100, empshare11plus=emp11plus/emp*100), by=.(year2,sic1)]

# Calculate total number of employees/total number of firms for each age category
dt.age_sic[, avgfsize := emp/firms]
dt.age_sic[, exitrate:= firmdeath_firms/firms*100]
dt.age_sic[fage4 == "a) 0", exitrate:= NA]

# Calculate Concentration by Age
# Break firms into age-size categories
temp <- dt.age_sz_sic[, .(year2, fage4, fsize, firms, emp, sector_name)]
# calculate number of firms size 250- by age and sector
temp.250minus <- temp[(fsize%in% szcat[1:6]), .(emp.250minus=sum(emp)), by=.(sector_name, year2, fage4)]
# calculate number of firms size 250+ by age
temp.250plus <- temp[!(fsize%in% szcat[1:6]), .(emp.250plus=sum(emp)), by=.(sector_name, year2, fage4)]
# combine with data on total firms
dt.age_sic <- merge(dt.age_sic, temp.250minus, by=c("year2", "fage4", "sector_name"), all=TRUE)
dt.age_sic <- merge(dt.age_sic, temp.250plus, by=c("year2", "fage4", "sector_name"), all=TRUE)
# dt.age <- merge(dt.age, temp.250plus, by=c("year2", "fage4"), all=TRUE)
# calculate share
dt.age_sic[, empshare250minus := emp.250minus/emp*100]
dt.age_sic[, empshare250plus := emp.250plus/emp*100]
dt.age_sic[, empshare250plus.imputed := 100 - empshare250minus]



dt.age0to25_sic <- dt.age_sic[!(fage4 == "g) 6 to 10" & year2 < 1987)]
dt.age0to25_sic <- dt.age0to25_sic[!(fage4 == "h) 11 to 15" & year2 < 1992)]
dt.age0to25_sic <- dt.age0to25_sic[!(fage4 == "i) 16 to 20" & year2 < 1997)]
dt.age0to25_sic <- dt.age0to25_sic[!(fage4 == "j) 21 to 25" & year2 < 2002)]
dt.age0to25_sic <- dt.age0to25_sic[!(fage4 == "k) 26+")]
dt.age0to25_sic <- dt.age0to25_sic[!(fage4 == "l) Left Censored")]



temp <- dt.age_sic[fage4 %in% c("k) 26+", "l) Left Censored")][year2 >= 2002, .(year2, sic1, fage4, firms, emp, avgfsize,
                                                                                exitrate, empshare250plus, empshare250plus.imputed)] 
                                                                    
temp[, firmwts := firms/sum(firms), by=.(year2,sic1)]
temp[, empwts := emp/sum(emp), by=.(year2,sic1)]
# Aggregate 26+ and Left Censored into a new dataset, which we will then merge into dt.agesic
temp2 <- temp[, .(firms=sum(firms), emp=sum(emp), 
                        avgfsize = weighted.mean(avgfsize, w=firmwts),
                        exitrate = weighted.mean(exitrate, w=firmwts),
                        empshare250plus = weighted.mean(empshare250plus, w=empwts),
                        empshare250plus.imputed = weighted.mean(empshare250plus.imputed, w=empwts)),
                    by=.(year2,sic1)]
temp2[, fage4:="k) Above 25"]
temp2 <- merge(temp2, temp.sic, by="sic1", all=TRUE)

temp3 <- merge(temp2, dt.age0to25_sic, all=TRUE, by=names(temp2))
# Drop all column names we never use. These have NAs anyway
dt.age0to25plus_sic <- temp3[, .(year2, sector_name, fage4, sic1, firms, emp, avgfsize, exitrate, empshare250plus, empshare250plus.imputed)]
setorder(dt.age0to25plus_sic, sector_name, year2, fage4)


fwrite(dt.sic, "./cleaned files/bds_f_sic_cleaned.csv")
fwrite(dt.age0to25plus_sic, "./cleaned files/bds_f_age0to25plus_sic_cleaned.csv")

