# ------------------------------------------------------------------------------
# File readme
# ------------------------------------------------------------------------------
# Figures 2 & 3 in Hopenhayn, Neira and Singhania (2022)
# Fig 2 shows average firm size, exit rate and concentration in the aggregate 
# Fig 3 shows the same variables broken down by firm age
# ------------------------------------------------------------------------------


rm(list = ls())
library(data.table)
library(ggplot2)
library(grid)
library(ggpubr)


fdir <- paste("./../figures/", sep='')

# Read cleaned data into the workspace
# Aggregate data
dt.all <- fread("./cleaned files/bds_f_all_cleaned.csv") 
# Data by firm age
dt.age0to25plus <- fread("./cleaned files/bds_f_age0to25plus_cleaned.csv")

# # ------------------------------------------------------------------------------
# Fig 2 - average firm size, exit rate and concentration in the aggregate 
# # ------------------------------------------------------------------------------

# Exit rate
p.exitrate <- ggplot(data=dt.all, aes(year2, exitrate/100)) + 
  geom_line(size=1.25, color="blue") + theme(aspect.ratio=1.2) +
  ylab('') +
  xlab('') +
  scale_y_continuous(limits=c(0.07, 0.108), labels = scales::percent_format(accuracy = 1)) +
  ggtitle('Exit Rate')


# Average firm size
p.avgfsize <- ggplot(data=dt.all, aes(year2, avgfsize)) + 
  geom_line(size=1.25, color="blue") + theme(aspect.ratio=1.2) +
  ylab('') +
  xlab('') +
  ggtitle('Average Firm Size')


# Concentration
p.empshare250plus <- ggplot(data=dt.all, aes(year2, empshare250plus/100))+ 
  geom_line(size=1.25, color="blue") + theme(aspect.ratio=1.2) +
  ylab('') +
  xlab('') +
  scale_y_continuous(limits=c(0.505,0.58), labels = scales::percent_format(accuracy=1)) +
  ggtitle('Concentration')

# Set common theme for the plots
theme_update(plot.title=element_text(hjust = 0.5), text=element_text(size=20))

# Combine aggregate plots to create a row with three plots
fname <- "Fig2.pdf"
g.avgfsize1 <- ggplotGrob(p.avgfsize)
g.exitrate1 <- ggplotGrob(p.exitrate)
g.empshare250plus1 <- ggplotGrob(p.empshare250plus)
p.agg <- cbind(g.exitrate1, g.avgfsize1, g.empshare250plus1,  size="first")
grid.newpage()
grid.draw(p.agg)
ggsave(p.agg, filename=paste(fdir,fname,sep=''), width=12, height=6)


# # ------------------------------------------------------------------------------
# # Fig 3 - Plot variables by firm age for ages 0 to 25+
# # ------------------------------------------------------------------------------

theme_update(plot.title=element_text(hjust = 0.5), text=element_text(size=28))

# Exit rate
p.exitrate.age0to25plus <- ggplot(data=dt.age0to25plus[!(fage4=="a) 0")], aes(year2, exitrate/100, colour=fage4)) +
  geom_line(size=1.25) + theme(aspect.ratio=1.2) +
  ylab('') +
  xlab('') +
  scale_y_continuous(limits=c(0.035, 0.27), breaks=c(0.05, 0.10, 0.15, 0.20, 0.25), labels = scales::percent_format(accuracy=1)) +
  scale_colour_discrete(name = "Firm age") +
  ggtitle('Exit Rate')

# Average firm size
p.avgfsize.age0to25plus <- ggplot(data=dt.age0to25plus, aes(year2, log(avgfsize), colour=fage4)) +
  geom_line(size=1.25) + theme(aspect.ratio=1.2) +
  ylab('') +
  xlab('') +
  scale_colour_discrete(name = "Firm age") +
  ggtitle('Log Average Firm Size')


# Concentration
p.empshare250plus.age0to25plus <- ggplot(data=dt.age0to25plus, aes(year2, empshare250plus.imputed/100, colour=fage4)) +
  geom_line(size=1.25) + theme(aspect.ratio=1.2) +
  ylab('') +
  xlab('') +
  scale_colour_discrete(name = "Firm age") +
  scale_y_continuous(limits=c(0, 0.80), breaks=c(0.0, 0.10, 0.20, 0.30, 0.40, 0.50, 0.60, 0.70, 0.80), labels = scales::percent_format(accuracy=1)) +
  ggtitle('Concentration')

# Set common theme for the plots
theme_update(plot.title=element_text(hjust = 0.5), text=element_text(size=20))

# Combine age plots to create a row with three plots
g.avgfsize.age0to25plus1 <- p.avgfsize.age0to25plus + guides(colour=FALSE)
g.exitrate.age0to25plus1 <- p.exitrate.age0to25plus + guides(colour=FALSE)
g.empshare250plus.age0to25plus1 <- p.empshare250plus.age0to25plus + guides(col=guide_legend(nrow=2, byrow=TRUE))

fname <- "Fig3.pdf"
p.age <- ggarrange( g.exitrate.age0to25plus1, g.avgfsize.age0to25plus1, g.empshare250plus.age0to25plus1, ncol=3, nrow=1, common.legend=TRUE, legend="bottom")
ggsave(p.age, filename=paste(fdir,fname,sep=''), width=12, height=6)


