#------------------------------------------------------------------------------------------------------
# Run regressions to isolate effect of age, sector and time on firm variables
# Independent variables: log average size, exit rate, and concenration
# Run year effects and then progressively control for age, sector and sector x age effects

#------------------------------------------------------------------------------------------------------
rm(list = ls())
library(data.table)
library(broom)



options(scipen=5)

# This function cleans up regression tables to extract year coefficients and put significance ***
cleanup_regtable <- function(regtab) {
  
  regtab <- tidy(regtab)
  results <- data.table()
  results <- rbind(results, regtab)
  
  
  results <- results[, estimate := round(estimate,4)]
  results <- results[, std.error := round(std.error,4)]
  results[p.value < 0.1, significance := "*"]
  results[p.value < 0.05, significance := "**"]
  results[p.value < 0.01, significance := "***"]
  # Get table ready for latex
  results[!is.na(significance), estimate2 := paste(estimate,significance,sep="")]
  results[is.na(significance), estimate2 := estimate]
  results[, std.error2 := paste0("(", std.error, ")")]
  
  cleanedup.tab <- results[term=="year2", .(estimate=estimate2, std.error=std.error2)]
  return(cleanedup.tab)
}


#------------------------------------------------------------------------------------------------------
# Read cleaned data on average size and exit rates by firm age and sector
#------------------------------------------------------------------------------------------------------
# AGGREGATE DATA
dt.all <- fread("./cleaned files/bds_f_all_cleaned.csv")
dt.all <- dt.all[year2 > 1977] # Drop 1977

# SECTOR DATA
dt.sic <- fread("./cleaned files/bds_f_sic_cleaned.csv")
dt.sic <- dt.sic[year2 > 1977] # Drop 1977
dt.sic[, firm_activity_wts := firms/sum(firms), by=.(year2)]
dt.sic[, emp_activity_wts := emp/sum(emp), by=.(year2)]
temp <- dt.sic[year2==1978, .(firm_activity_wts_sic1978=firm_activity_wts, emp_activity_wts_sic1978=emp_activity_wts), by=.(sic1)]
dt.sic <- merge(dt.sic, temp, by=c("sic1"))

# AGE DATA
dt.age <- fread("./cleaned files/bds_f_age0to25plus_cleaned.csv")
dt.age <- dt.age[year2 > 1977] # Drop 1977
dt.age[, inv_numobs_wts:=1/.N, by=.(fage4)]
# Add 2014 firm and employment activity wts
dt.age[, firm_activity_wts := firms/sum(firms), by=.(year2)]
dt.age[, emp_activity_wts := emp/sum(emp), by=.(year2)]


temp <- dt.age[year2==2014, .(firm_activity_wts2014=firm_activity_wts, emp_activity_wts2014=emp_activity_wts), by=.(fage4)]
dt.age <- merge(dt.age, temp, by=c("fage4"))
# 2014 firms wts * equally weighted age groups
dt.age[, firm_activity_wts2014_equalwtsagegrp:=firm_activity_wts2014*inv_numobs_wts]
dt.age[, emp_activity_wts2014_equalwtsagegrp:=emp_activity_wts2014*inv_numobs_wts]

# AGE X SECTOR DATA
dt.age_sic <- fread("./cleaned files/bds_f_age0to25plus_sic_cleaned.csv")
dt.age_sic <- dt.age_sic[year2 > 1977] # Drop 1977
# Specify different weights in order to run weighted regressions
# weight by inverse of number of observations
dt.age_sic[, inv_numobs_wts:=1/.N, by=.(sic1, fage4)]
# 2014 firms wts
dt.age_sic[, firm_activity_wts := firms/sum(firms), by=.(year2, sic1)]
dt.age_sic[, emp_activity_wts := emp/sum(emp), by=.(year2, sic1)]

temp <- dt.age_sic[year2==2014, .(firm_activity_wts2014=firm_activity_wts, emp_activity_wts2014=emp_activity_wts), by=.(sic1,fage4)]
dt.age_sic <- merge(dt.age_sic, temp, by=c("sic1", "fage4"))
# 2014 firms wts * equally weighted age groups
dt.age_sic[, firm_activity_wts2014_equalwtsagegrp:=firm_activity_wts2014*inv_numobs_wts]
dt.age_sic[, emp_activity_wts2014_equalwtsagegrp:=emp_activity_wts2014*inv_numobs_wts]

dt.age_sic[, equal_wts := 1]
dt.age_sic[, firms_sic := sum(firms), by=.(year2,sic1)]
dt.age_sic[, emp_sic := sum(emp), by=.(year2,sic1)]
dt.age_sic[, firm_wts_sic := firms_sic/sum(firms), by=.(year2)]
dt.age_sic[, emp_wts_sic := emp_sic/sum(emp), by=.(year2)]
dt.age_sic[, firm_activity_wts_sic := equal_wts*firm_wts_sic]
dt.age_sic[, emp_activity_wts_sic := equal_wts*emp_wts_sic]
temp <- dt.age_sic[year2==1978 & fage4=="a) 0", .(firm_activity_wts_sic1978=firm_activity_wts_sic, emp_activity_wts_sic1978=emp_activity_wts_sic), by=.(sic1)]

dt.age_sic <- merge(dt.age_sic, temp, by=c("sic1"))

dt.age_sic[empshare250plus.imputed<0.00000001, empshare250plus.imputed:=NA]


# --------------------------------------------------------------------------------------
# Average firm size regressions
# --------------------------------------------------------------------------------------
avgfsize.regmodel1 <- lm(avgfsize ~ year2, data=dt.all, na.action=na.omit)
avgfsize.regmodel2 <- lm(avgfsize ~ year2 + factor(sector_name) , data=dt.sic, weights=firm_activity_wts_sic1978, na.action=na.omit)
avgfsize.regmodel3 <- lm(avgfsize ~ year2 + factor(fage4) , data=dt.age, weights=firm_activity_wts2014_equalwtsagegrp, na.action=na.omit)
avgfsize.regmodel4 <- lm(avgfsize ~ year2 + factor(fage4) + factor(sector_name), data=dt.age_sic, weights=firm_activity_wts_sic1978*firm_activity_wts2014_equalwtsagegrp, na.action=na.omit)
avgfsize.regmodel5 <- lm(avgfsize ~ year2 + factor(fage4) + factor(sector_name) + factor(fage4):factor(sector_name), data=dt.age_sic, weights=firm_activity_wts_sic1978*firm_activity_wts2014_equalwtsagegrp, na.action=na.omit)

# Collect results
avgfsize.results <- rbind(cleanup_regtable(avgfsize.regmodel1), cleanup_regtable(avgfsize.regmodel2), 
                          cleanup_regtable(avgfsize.regmodel3), cleanup_regtable(avgfsize.regmodel4), 
                          cleanup_regtable(avgfsize.regmodel5))

# Collect R-squared of each regression, along with the number of observations
avgfsize.results.numobs <- c(nobs(avgfsize.regmodel1), nobs(avgfsize.regmodel2),
                             nobs(avgfsize.regmodel3), nobs(avgfsize.regmodel4),
                             nobs(avgfsize.regmodel5))
avgfsize.results.R2 <- c(summary(avgfsize.regmodel1)$r.squared, summary(avgfsize.regmodel2)$r.squared,
                         summary(avgfsize.regmodel3)$r.squared, summary(avgfsize.regmodel4)$r.squared,
                         summary(avgfsize.regmodel5)$r.squared)
avgfsize.results.R2 <- round(avgfsize.results.R2, digits=3)
avgfsize.results <- cbind(avgfsize.results, R2=avgfsize.results.R2, numobs=avgfsize.results.numobs)




# --------------------------------------------------------------------------------------
# Exit rate regressions
# --------------------------------------------------------------------------------------

exitrate.regmodel1 <- lm(exitrate ~ year2, data=dt.all, na.action=na.omit)
exitrate.regmodel2 <- lm(exitrate ~ year2 + factor(sector_name), data=dt.sic,weights=firm_activity_wts_sic1978, na.action=na.omit)
exitrate.regmodel3 <- lm(exitrate ~ year2 + factor(fage4), data=dt.age[fage4 != "a) 0"], weights=firm_activity_wts2014_equalwtsagegrp, na.action=na.omit)
exitrate.regmodel4 <- lm(exitrate ~ year2 + factor(fage4) + factor(sector_name) , data=dt.age_sic[fage4 != "a) 0"] , weights=firm_activity_wts_sic1978*firm_activity_wts2014_equalwtsagegrp, na.action=na.omit)
exitrate.regmodel5 <- lm(exitrate ~ year2  + factor(fage4) + factor(sector_name) + factor(fage4):factor(sector_name), data=dt.age_sic[fage4 != "a) 0"] , weights=firm_activity_wts_sic1978*firm_activity_wts2014_equalwtsagegrp, na.action=na.omit)

# Collect results
exitrate.results <- rbind(cleanup_regtable(exitrate.regmodel1), cleanup_regtable(exitrate.regmodel2), 
                          cleanup_regtable(exitrate.regmodel3), cleanup_regtable(exitrate.regmodel4), 
                          cleanup_regtable(exitrate.regmodel5))

# Collect R-squared of each regression, along with the number of observations
exitrate.results.numobs <- c(nobs(exitrate.regmodel1), nobs(exitrate.regmodel2),
                             nobs(exitrate.regmodel3), nobs(exitrate.regmodel4),
                             nobs(exitrate.regmodel5))
exitrate.results.R2 <- c(summary(exitrate.regmodel1)$r.squared, summary(exitrate.regmodel2)$r.squared,
                         summary(exitrate.regmodel3)$r.squared, summary(exitrate.regmodel4)$r.squared,
                         summary(exitrate.regmodel5)$r.squared)
exitrate.results.R2 <- round(exitrate.results.R2, digits=3)
exitrate.results <- cbind(exitrate.results, R2=exitrate.results.R2, numobs=exitrate.results.numobs)


# --------------------------------------------------------------------------------------
# Concentration regressions (Employment share of 250+ firms)
# --------------------------------------------------------------------------------------
empshare250plus.regmodel1 <- lm(empshare250plus ~ year2, data=dt.all, na.action=na.omit)
empshare250plus.regmodel2 <- lm(empshare250plus ~ year2 + factor(sector_name), data=dt.sic,weights=emp_activity_wts_sic1978, na.action=na.omit)
empshare250plus.regmodel3 <- lm(empshare250plus.imputed ~ year2 + factor(fage4), data=dt.age, weights=firm_activity_wts2014_equalwtsagegrp, na.action=na.omit)
empshare250plus.regmodel4 <- lm(empshare250plus.imputed ~ year2 + factor(fage4) + factor(sector_name) , data=dt.age_sic , weights=emp_activity_wts_sic1978*firm_activity_wts2014_equalwtsagegrp, na.action=na.omit)
empshare250plus.regmodel5 <- lm(empshare250plus.imputed ~ year2  + factor(fage4) + factor(sector_name) + factor(fage4):factor(sector_name), data=dt.age_sic, weights=emp_activity_wts_sic1978*firm_activity_wts2014_equalwtsagegrp, na.action=na.omit)

# Collect results
empshare250plus.results <- rbind(cleanup_regtable(empshare250plus.regmodel1), cleanup_regtable(empshare250plus.regmodel2), 
                          cleanup_regtable(empshare250plus.regmodel3), cleanup_regtable(empshare250plus.regmodel4), 
                          cleanup_regtable(empshare250plus.regmodel5))

# Collect R-squared of each regression, along with the number of observations
empshare250plus.results.numobs <- c(nobs(empshare250plus.regmodel1), nobs(empshare250plus.regmodel2),
                             nobs(empshare250plus.regmodel3), nobs(empshare250plus.regmodel4),
                             nobs(empshare250plus.regmodel5))
empshare250plus.results.R2 <- c(summary(empshare250plus.regmodel1)$r.squared, summary(empshare250plus.regmodel2)$r.squared,
                         summary(empshare250plus.regmodel3)$r.squared, summary(empshare250plus.regmodel4)$r.squared,
                         summary(empshare250plus.regmodel5)$r.squared)
empshare250plus.results.R2 <- round(empshare250plus.results.R2, digits=3)


empshare250plus.results <- cbind(empshare250plus.results, R2=empshare250plus.results.R2, numobs=empshare250plus.results.numobs)

# ------------------------------------
# Table 1:  Time trends
# ------------------------------------

print(exitrate.results)
print(avgfsize.results)
print(empshare250plus.results)
