#------------------------------------------------------------------------------------------------------
# Calculate time trends in the aggregate and by firm age in each sector
#------------------------------------------------------------------------------------------------------
rm(list = ls())
library(data.table)
library(broom)
library(tidyr)
library(dplyr)


options(scipen=5)


#------------------------------------------------------------------------------------------------------
# Read cleaned data on average size and exit rates by firm age and sector
#------------------------------------------------------------------------------------------------------
dt.all <- fread("./cleaned files/bds_f_all_cleaned.csv")
dt.all <- dt.all[year2 > 1977]

dt.sic <- fread("./cleaned files/bds_f_sic_cleaned.csv")
dt.sic <- dt.sic[year2 > 1977]


# Write function to run aggregate regressions
run_agg_regs <- function(y, dt)
{
  results <- data.table()
  regmodel <- lm(get(y) ~ year2, data=dt, na.action = na.omit)
  temp <- tidy(regmodel)
  results <- rbind(results, temp)
  results[p.value < 0.1, significance := "*"]
  results[p.value < 0.05, significance := "**"]
  results[p.value < 0.01, significance := "***"]
  results[, sector_name := "Aggregate"]
  return(results)
}


# Write function to run sectoral regressions
run_sectoral_regs <- function(y, dt)
{
  sector_names <- dt[, unique(sector_name)]
  results <- data.table()
  for (s in sector_names) {
      regmodel <- lm(get(y) ~ year2, data=dt[sector_name %in% s], na.action = na.omit)
      temp <- tidy(regmodel) %>% mutate(sector_name = s)
      results <- rbind(results, temp)
  }
  results[p.value < 0.1, significance := "*"]
  results[p.value < 0.05, significance := "**"]
  results[p.value < 0.01, significance := "***"]
  return(results)
}

# Housekeeping on output
cleanup_table <- function(regtab)
{
  regtab <- regtab[order(sector_name)]
  regtab <- regtab[, estimate := round(estimate,4)]
  regtab <- regtab[, std.error := round(std.error,4)]
  return(regtab)
}


# Run aggregate regressions
aggtrend.entryrate <- run_agg_regs("entry_rate", dt=dt.all)
aggtrend.entryrate.results <- aggtrend.entryrate[term == "year2", .(estimate, std.error, significance, sector_name)]

aggtrend.firmshare11plus <- run_agg_regs("firmshare11plus", dt=dt.all[year2 > 1986])
aggtrend.firmshare11plus.results <- aggtrend.firmshare11plus[term == "year2", .(estimate, std.error, significance, sector_name)]


# Omit Agriculture and Mining
`%notin%` <- Negate(`%in%`)
dt.sic <- dt.sic[sector_name %notin% c("Agriculture", "Mining")]


# Run sectoral regressions
# entry rate
entry_rate.trend <- run_sectoral_regs("entry_rate", dt=dt.sic)
entry_rate.results <- entry_rate.trend[term == "year2", .(estimate, std.error, significance), by=sector_name]
entry_rate.results <- merge(aggtrend.entryrate.results, entry_rate.results, all=TRUE)
entry_rate.results <- cleanup_table(entry_rate.results)

# firmshare 11+
firmshare11plus.trend <- run_sectoral_regs("firmshare11plus", dt=dt.sic[year2 > 1986])
firmshare11plus.results <- firmshare11plus.trend[term == "year2", .(estimate, std.error, significance), by=sector_name]
firmshare11plus.results <- merge(aggtrend.firmshare11plus.results, firmshare11plus.results, all=TRUE)
firmshare11plus.results <- cleanup_table(firmshare11plus.results)

# ----------------------------------
# Table 2 Panel A - Entry and Firm Aging by Sector
print(entry_rate.results[, .SD, by=sector_name])
print(firmshare11plus.results[, .SD, by=sector_name])




