#------------------------------------------------------------------------------------------------------
# Calculate time trends in the aggregate and by firm age in each sector
#------------------------------------------------------------------------------------------------------
rm(list = ls())
library(data.table)
library(broom)
library(tidyr)
library(dplyr)


options(scipen=5)

#------------------------------------------------------------------------------------------------------
# Read cleaned data on average size and exit rates by firm age and sector
#------------------------------------------------------------------------------------------------------
dt.sic <- fread("./cleaned files/bds_f_sic_cleaned.csv")
dt.sic <- dt.sic[year2 > 1977]


dt.age_sic <- fread("./cleaned files/bds_f_age0to25plus_sic_cleaned.csv")
dt.age_sic <- dt.age_sic[year2 > 1977]
# Specify different weights in order to run weighted regressions
# weight by inverse of number of observations
dt.age_sic[, inv_numobs_wts:=1/.N, by=.(sic1, fage4)]
# 2014 firms wts
dt.age_sic[, firmshares_agegrp := firms/sum(firms), by=.(year2, sic1)]
dt.age_sic[, firm_activity_wts := firmshares_agegrp]
dt.age_sic[, empshares_agegrp := emp/sum(emp), by=.(year2, sic1)]
dt.age_sic[, emp_activity_wts := empshares_agegrp]


temp <- dt.age_sic[year2==2014, .(firm_activity_wts2014=firm_activity_wts, emp_activity_wts2014=emp_activity_wts), by=.(sic1,fage4)]
dt.age_sic <- merge(dt.age_sic, temp, by=c("sic1", "fage4"))
# 2014 firms wts * equally weighted age groups
dt.age_sic[, firm_activity_wts2014_equalwtsagegrp:=firm_activity_wts2014*inv_numobs_wts]
dt.age_sic[, emp_activity_wts2014_equalwtsagegrp:=emp_activity_wts2014*inv_numobs_wts]


# Write another function
run_sectoral_regs <- function(y, dt)
{
  sector_names <- dt[, unique(sector_name)]
  results <- data.table()
  for (s in sector_names) {
      regmodel <- lm(get(y) ~ year2, data=dt[sector_name %in% s], na.action = na.omit)
      temp <- tidy(regmodel) %>% mutate(sector_name = s)
      results <- rbind(results, temp)
  }
  # print(results)
  results[p.value < 0.1, significance := "*"]
  results[p.value < 0.05, significance := "**"]
  results[p.value < 0.01, significance := "***"]
  return(results)
}

run_sectoral_regs_agectrl <- function(y, dt, wts)
{
  sector_names <- dt[, unique(sector_name)]
  results <- data.table()
  for (s in sector_names) {
    regmodel <- lm(get(y) ~ year2 + factor(fage4), data=dt[sector_name %in% s], weights=get(wts), na.action = na.omit)
    temp <- tidy(regmodel) %>% mutate(sector_name = s)
    results <- rbind(results, temp)
  }
  # print(results)
  results[p.value < 0.1, significance := "*"]
  results[p.value < 0.05, significance := "**"]
  results[p.value < 0.01, significance := "***"]
  return(results)
}


# Omit Agriculture and Mining
`%notin%` <- Negate(`%in%`)
dt.sic <- dt.sic[sector_name %notin% c("Agriculture", "Mining")]
dt.age_sic <- dt.age_sic[sector_name %notin% c("Agriculture", "Mining")]


# Sectoral trends
# Run regressions
avgfsize.trend <- run_sectoral_regs("avgfsize", dt=dt.sic)
exitrate.trend <- run_sectoral_regs("exitrate", dt=dt.sic)
conc.trend <- run_sectoral_regs("empshare250plus", dt=dt.sic)

# Sectoral trends by age
avgfsize.trend.agectrl <- run_sectoral_regs_agectrl("avgfsize", dt=dt.age_sic, wts="firm_activity_wts2014_equalwtsagegrp")
exitrate.trend.agectrl <- run_sectoral_regs_agectrl("exitrate", dt=dt.age_sic[!fage4 == "a) 0"], wts="firm_activity_wts2014_equalwtsagegrp")
conc.trend.agectrl <- run_sectoral_regs_agectrl("empshare250plus.imputed", dt=dt.age_sic, wts="firm_activity_wts2014_equalwtsagegrp")



# Clean up output and make regression tables 
# -----------------------------------------------------------------------
# AVERAGE FIRM SIZE
# -----------------------------------------------------------------------
avgfsize.trend <- avgfsize.trend[term == "year2", .(sector_name, estimate)]
avgfsize.trend.agectrl <- avgfsize.trend.agectrl[term == "year2", .(sector_name, estimate)]
setnames(avgfsize.trend.agectrl, "estimate", "estimate.agectrl")

avgfsize.1978levels <- dt.sic[year2==1978, avgfsize, by=sector_name]
setnames(avgfsize.1978levels, "avgfsize", "avgfsize1978")

avgfsize.results <- merge(avgfsize.trend, avgfsize.trend.agectrl)
avgfsize.results <- merge(avgfsize.results, avgfsize.1978levels)
avgfsize.results[, avgfsize_aging_only:= round((estimate - estimate.agectrl)*36/avgfsize1978*100), by=sector_name]



# -----------------------------------------------------------------------
# EXIT RATE
# -----------------------------------------------------------------------
exitrate.trend <- exitrate.trend[term == "year2", .(sector_name, estimate)]
exitrate.trend.agectrl <- exitrate.trend.agectrl[term == "year2", .(sector_name, estimate)]
setnames(exitrate.trend.agectrl, "estimate", "estimate.agectrl")

exitrate.1978levels <- dt.sic[year2==1978, exitrate, by=sector_name]
setnames(exitrate.1978levels, "exitrate", "exitrate1978")

exitrate.results <- merge(exitrate.trend, exitrate.trend.agectrl)
exitrate.results <- merge(exitrate.results, exitrate.1978levels)
exitrate.results[, exitrate_aging_only:= round((estimate - estimate.agectrl)*36/exitrate1978*100), by=sector_name]


# -----------------------------------------------------------------------
# CONCENTRATION
# -----------------------------------------------------------------------
conc.trend <- conc.trend[term == "year2", .(sector_name, estimate)]
conc.trend.agectrl <- conc.trend.agectrl[term == "year2", estimate, by=sector_name]
setnames(conc.trend.agectrl, "estimate", "estimate.agectrl")

conc.1978levels <- dt.sic[year2==1978 & sector_name != "Services", empshare250plus, by=sector_name]
# Services is missing data for 1978, so use 1979 instead
conc.1979levels.services <- dt.sic[year2==1979 & sector_name =="Services", empshare250plus,by=sector_name]
conc.1978levels <- merge(conc.1978levels, conc.1979levels.services,all=TRUE)
setnames(conc.1978levels, "empshare250plus", "empshare250plus1978")

conc.results <- merge(conc.trend, conc.trend.agectrl)
conc.results <- merge(conc.results, conc.1978levels)
conc.results[, conc_aging_only:= round((estimate - estimate.agectrl)*36/empshare250plus1978*100), by=sector_name]




# Combine results into one table
results_aging_only <- merge(exitrate.results[, exitrate_aging_only, by=sector_name], avgfsize.results[, avgfsize_aging_only, by=sector_name])
results_aging_only <- merge(results_aging_only, conc.results[,conc_aging_only, by=sector_name])

# ----------------------------------
# Table 2 Panel B - Entry and Firm Aging by Sector
print(results_aging_only)


